/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.filing.filingImages.service;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.core.web.Servlets;
import com.jeeplus.modules.filing.approvalStage.entity.ApprovalStage;
import com.jeeplus.modules.filing.approvalStage.entity.EarlyStage;
import com.jeeplus.modules.filing.approvalStage.entity.StudyStage;
import com.jeeplus.modules.filing.approvalStage.mapper.ApprovalStageMapper;
import com.jeeplus.modules.filing.approvalStage.mapper.EarlyStageMapper;
import com.jeeplus.modules.filing.approvalStage.mapper.StudyStageMapper;
import com.jeeplus.modules.filing.basicInformation.entity.BasicInformation;
import com.jeeplus.modules.filing.basicInformation.mapper.BasicInformationMapper;
import com.jeeplus.modules.filing.filingImages.entity.FilingImages;
import com.jeeplus.modules.filing.filingImages.mapper.FilingImagesMapper;
import com.jeeplus.modules.filing.filingImages.utils.ZipCompressUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * 图片管理Service
 *
 * @author qy
 * @version 2021-03-25
 */
@Service
@Transactional(readOnly = true)
public class FilingImagesService extends CrudService<FilingImagesMapper, FilingImages> {

    @Autowired
    private BasicInformationMapper basicInformationMapper;


    @Autowired
    private ApprovalStageMapper approvalStageMapper;


    @Autowired
    private EarlyStageMapper earlyStageMapper;

    @Autowired
    private StudyStageMapper studyStageMapper;

    public List<FilingImages> findByProjectId(String id) {
        return mapper.findByProjectId(id);
    }



    /**
     * 通过项目id导出项目的核准附件
     *
     * @param idList
     */
    public String exportApproved(List<String> idList) throws Exception {
        //设置下载的压缩包名（固定字符+时间戳）
        long timeMillis = System.currentTimeMillis();
        String fileLocality = "项目附件_" + timeMillis;
        //判定当前系统
        String path = Global.getAttachmentDir2();
        path = path + "/" + fileLocality;
        //检查该路径对应的目录是否存在. 如果不存在则创建目录
        File dir = new File(path);
        if (!dir.exists()) {
            dir.mkdirs();
        }
        //通过id查询出所有的项目信息
        List<ApprovalStage> byIdList = approvalStageMapper.findByIdList(idList);
        //循环遍历，判断项目是否存在
        for (ApprovalStage a :
                byIdList) {
            File dirFile = null;
            List<FilingImages> filingImagesList = null;
            List<FilingImages> exportList = new ArrayList<>();
            String downloadPath = path + "/" + a.getProjectCode();
            String approvedAnnex = a.getApprovedAnnex();
            if (null!=approvedAnnex && !"".equals(approvedAnnex)) {
                dirFile = new File(downloadPath + "/核准阶段");
                if (approvedAnnex.contains("|")) {
                    String[] split = approvedAnnex.split("\\|");
                    for (int i = 0; i < split.length; i++) {
                        FilingImages filingImages = new FilingImages();
                        filingImages.setImgPath(split[i]);
                        filingImages.setProjectCode(a.getProjectCode());
                        filingImages.setPurpose("核准附件");
                        exportList.add(filingImages);
                    }
                } else {
                    FilingImages filingImages = new FilingImages();
                    filingImages.setImgPath(approvedAnnex);
                    filingImages.setProjectCode(a.getProjectCode());
                    filingImages.setPurpose("核准附件");
                    exportList.add(filingImages);
                }
                if (exportList.size() > 0) {
                    this.disposeFileList(exportList, dirFile.getPath());
                }
            }
        }
        //处理完之后进行打包压缩并删除之前的文件
        ZipCompressUtil zipUtil = new ZipCompressUtil();
        File resultFile = new File(path);
        File file = zipUtil.zipCompress(resultFile, true);
        return file.getPath();
    }


    /**
     * 项目相应文件批量下载并压缩
     *
     * @param idList
     */
    public String exportStudy(List<String> idList) throws Exception {
        //设置下载的压缩包名（固定字符+时间戳）
        long timeMillis = System.currentTimeMillis();
        String fileLocality = "项目附件_" + timeMillis;
        //判定当前系统
        String path = Global.getAttachmentDir2();
        path = path + "/" + fileLocality;
        //检查该路径对应的目录是否存在. 如果不存在则创建目录
        File dir = new File(path);
        if (!dir.exists()) {
            dir.mkdirs();
        }

        List<StudyStage> allList = new ArrayList<>();
        //通过id查询出所有的项目信息
        List<StudyStage> byIdList = studyStageMapper.findAllStudy(idList);

        for (StudyStage a :
                byIdList) {
            allList.add(a);
            List<StudyStage> childList = studyStageMapper.findChildList(a);
            if (childList.size() > 0) {
                for (StudyStage b :
                        childList) {
                    allList.add(b);
                }
            }
        }

        //循环遍历，判断项目是否存在
        for (StudyStage a :
                allList) {
            File dirFile = null;
            List<FilingImages> filingImagesList = null;
            List<FilingImages> exportList = new ArrayList<>();
            String downloadPath = path + "/" + a.getProjectCode();
            String studyAppendix = a.getStudyAppendix();
            if (null!=studyAppendix && !"".equals(studyAppendix)) {
                dirFile = new File(downloadPath + "/可研阶段");
                if (studyAppendix.contains("|")) {
                    String[] split = studyAppendix.split("\\|");
                    for (int i = 0; i < split.length; i++) {
                        FilingImages filingImages = new FilingImages();
                        filingImages.setImgPath(split[i]);
                        filingImages.setProjectCode(a.getProjectCode());
                        filingImages.setPurpose("可研附件");
                        exportList.add(filingImages);
                    }
                } else {
                    FilingImages filingImages = new FilingImages();
                    filingImages.setImgPath(studyAppendix);
                    filingImages.setProjectCode(a.getProjectCode());
                    filingImages.setPurpose("可研附件");
                    exportList.add(filingImages);
                }
                if (exportList.size() > 0) {
                    this.disposeFileList(exportList, dirFile.getPath());
                }
            }
        }
        //处理完之后进行打包压缩并删除之前的文件
        ZipCompressUtil zipUtil = new ZipCompressUtil();
        File resultFile = new File(path);
        File file = zipUtil.zipCompress(resultFile, true);
        return file.getPath();
    }


    /**
     * 项目相应文件批量下载并压缩
     *
     * @param idList
     */
    public String exportStudyStage(List<String> idList,String purpose) throws Exception {
        //设置下载的压缩包名（固定字符+时间戳）
        long timeMillis = System.currentTimeMillis();
        String fileLocality = "项目附件_" + timeMillis;
        //判定当前系统
        String path = Global.getAttachmentDir2();
        path = path + "/" + fileLocality;
        //检查该路径对应的目录是否存在. 如果不存在则创建目录
        File dir = new File(path);
        if (!dir.exists()) {
            dir.mkdirs();
        }
        List<String> allIdList = new ArrayList<>();
        //通过id查询出
        //通过id查询出所有有关联的id
        for (String projectId : idList) {
            allIdList.add(projectId);
            BasicInformation one = basicInformationMapper.findOne(projectId);
            List<BasicInformation> childByCode = null;
            try {
                childByCode = basicInformationMapper.findChildByCode(one.getProjectCode());
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (BasicInformation b :
                    childByCode) {
                allIdList.add(b.getId());
            }
        }

        //遍历项目id并查询对应数据信息
        for (String projectId : allIdList) {
            //通过id查询项目信息
            BasicInformation one = basicInformationMapper.findOne(projectId);
            String projectCode = one.getProjectCode();
            //通过项目信息获取附件信息
            List<String> byCode = mapper.findAllPurpose(projectCode);
            File dirFile = null;
            List<FilingImages> filingImagesList = null;
            List<FilingImages> exportList = new ArrayList<>();
            String downloadPath = path + "/" + one.getProjectCode();
            for (String a :
                    byCode) {
                switch (a) {
                    case "核准附件":
                        dirFile = new File(downloadPath + "/核准阶段");
                        filingImagesList = Lists.newArrayList();
                        filingImagesList = mapper.findAllUpload(one.getProjectCode(), a);
                        for (FilingImages filing :
                                filingImagesList) {
                            if (filing.getImgPath().contains("|")) {
                                String[] split = filing.getImgPath().split("\\|");
                                for (int i = 0; i < split.length; i++) {
                                    FilingImages filingImages = new FilingImages();
                                    filingImages.setImgPath(split[i]);
                                    filingImages.setProjectCode(filing.getProjectCode());
                                    filingImages.setPurpose(filing.getPurpose());
                                    exportList.add(filingImages);
                                }

                            } else {
                                exportList.add(filing);
                            }
                        }
                        this.disposeFileList(exportList, dirFile.getPath());
                        break;
                    case "可研附件":
                        dirFile = new File(downloadPath + "/可研阶段");
                        filingImagesList = Lists.newArrayList();
                        filingImagesList = mapper.findAllUpload(one.getProjectCode(), a);
                        for (FilingImages filing :
                                filingImagesList) {
                            if (filing.getImgPath().contains("|")) {
                                String[] split = filing.getImgPath().split("\\|");
                                for (int i = 0; i < split.length; i++) {
                                    FilingImages filingImages = new FilingImages();
                                    filingImages.setImgPath(split[i]);
                                    filingImages.setProjectCode(filing.getProjectCode());
                                    filingImages.setPurpose(filing.getPurpose());
                                    exportList.add(filingImages);
                                }

                            } else {
                                exportList.add(filing);
                            }
                        }
                        this.disposeFileList(exportList, dirFile.getPath());
                        break;
                    case "初设附件":
                        dirFile = new File(downloadPath + "/初设阶段");
                        break;
                }
            }

        }
        //处理完之后进行打包压缩并删除之前的文件
        ZipCompressUtil zipUtil = new ZipCompressUtil();
        File resultFile = new File(path);
        File file = zipUtil.zipCompress(resultFile, true);
        return file.getPath();
    }


    /**
     * 处理文件信息
     *
     * @param downloadPath
     */
    public void disposeFileList(List<FilingImages> filingImages, String downloadPath) throws IOException {
        ZipCompressUtil zipCompressUtil = new ZipCompressUtil();
        for (FilingImages info : filingImages) {
            zipCompressUtil.downloadRuralProject(info.getImgPath().substring(0, info.getImgPath().lastIndexOf("/")), info.getImgPath().substring(info.getImgPath().lastIndexOf("/") + 1, info.getImgPath().length()), downloadPath);
        }
    }

    /**
     * 下载压缩文件
     *
     * @param filePath
     * @param response
     */
    public void downloadZipFile(String filePath, HttpServletResponse response) {
        File file = new File(filePath);
        // 取得文件名。
        String fileName = file.getName();
        InputStream in = null;
        try {
            fileName = URLEncoder.encode(fileName, "UTF8");
            in = new FileInputStream(file);
            response.reset();//重置 响应头
            response.setCharacterEncoding("UTF-8");
            response.setContentType("application/x-download");
            response.setHeader("Content-disposition", "attachment; filename=" + fileName);

            byte[] b = new byte[1024];
            int len;
            while ((len = in.read(b)) > 0) {
                response.getOutputStream().write(b, 0, len);
            }
            in.close();
            response.getOutputStream().close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            //路径是个文件且不为空时删除文件
            if (file.isFile() && file.exists()) {
                file.delete();
            }
        }
    }


    /**
     * 通过项目id导出项目的核准附件
     *
     * @param idList
     */
    public String exportEarlyStage(List<String> idList) throws Exception {
        //设置下载的压缩包名（固定字符+时间戳）
        long timeMillis = System.currentTimeMillis();
        String fileLocality = "项目附件_" + timeMillis;
        //判定当前系统
        String path = Global.getAttachmentDir2();
        path = path + "/" + fileLocality;
        //检查该路径对应的目录是否存在. 如果不存在则创建目录
        File dir = new File(path);
        if (!dir.exists()) {
            dir.mkdirs();
        }

        List<EarlyStage> allList = new ArrayList<>();
        //通过id查询出所有的项目信息
        List<EarlyStage> byIdList = earlyStageMapper.findAllStudy(idList);

        for (EarlyStage a :
                byIdList) {
            allList.add(a);
            List<EarlyStage> childList = earlyStageMapper.findChildList(a);
            if (childList.size() > 0) {
                for (EarlyStage b :
                        childList) {
                    allList.add(b);
                }
            }
        }
        //循环遍历，判断项目是否存在
        for (EarlyStage a :
                allList) {
            File dirFile = null;
            File dirFile2 = null;
            List<FilingImages> filingImagesList = null;
            List<FilingImages> exportList = new ArrayList<>();
            List<FilingImages> exportList2 = new ArrayList<>();
            String downloadPath = path + "/" + a.getProjectCode();
            String approvedAnnex = a.getApprovedAnnex();
            if (null!=approvedAnnex && !"".equals(approvedAnnex)) {
                dirFile = new File(downloadPath + "/核准阶段");
                if (approvedAnnex.contains("|")) {
                    String[] split = approvedAnnex.split("\\|");
                    for (int i = 0; i < split.length; i++) {
                        FilingImages filingImages = new FilingImages();
                        filingImages.setImgPath(split[i]);
                        filingImages.setProjectCode(a.getProjectCode());
                        filingImages.setPurpose("核准附件");
                        exportList.add(filingImages);
                    }
                } else {
                    FilingImages filingImages = new FilingImages();
                    filingImages.setImgPath(approvedAnnex);
                    filingImages.setProjectCode(a.getProjectCode());
                    filingImages.setPurpose("核准附件");
                    exportList.add(filingImages);
                }
            }
            String studyAppendix = a.getStudyAppendix();
            if (null!=studyAppendix && !"".equals(studyAppendix)) {
                dirFile2 = new File(downloadPath + "/可研阶段");
                if (studyAppendix.contains("|")) {
                    String[] split = studyAppendix.split("\\|");
                    for (int i = 0; i < split.length; i++) {
                        FilingImages filingImages = new FilingImages();
                        filingImages.setImgPath(split[i]);
                        filingImages.setProjectCode(a.getProjectCode());
                        filingImages.setPurpose("核准附件");
                        exportList2.add(filingImages);
                    }
                } else {
                    FilingImages filingImages = new FilingImages();
                    filingImages.setImgPath(studyAppendix);
                    filingImages.setProjectCode(a.getProjectCode());
                    filingImages.setPurpose("可研附件");
                    exportList2.add(filingImages);
                }
            }
            if (exportList.size() > 0) {
                this.disposeFileList(exportList, dirFile.getPath());
            }
            if (exportList2.size() > 0) {
                this.disposeFileList(exportList2, dirFile2.getPath());
            }
        }
        //处理完之后进行打包压缩并删除之前的文件
        ZipCompressUtil zipUtil = new ZipCompressUtil();
        File resultFile = new File(path);
        File file = zipUtil.zipCompress(resultFile, true);
        return file.getPath();
    }

}